
#include <pololu/orangutan.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include "hc04.h"

// Init the constants

int hc04_echo_count = 0;
uint16_t time_us = 0;
uint16_t total_distance = 0;
uint16_t hc04_distance = 0;

// Functions

void hc04_init() {

    DDRD |=  (1 << 1);     // PD1 trigger - output
    DDRD &= ~(1 << 0) ;    // PD0 echo - input

    PORTD &= ~(1<<1);      // set trigger low

    PRR &= ~(1<<PRTIM1);   // Activate timer1 module
    TCNT1 = 0;	           // Initial timer value

    TCCR1B |=  (1<<CS10);  // set timer prescaler ( /64 )
    TCCR1B |=  (1<<CS11); 
    TCCR1B &= ~(1<<CS12);

    PCICR =  (1<<PCIE2);   // Pin change on Port D
    PCMSK2 = (1<<PCINT16); // Monitor PIND0 for pin change

    sei(); // Enable Global Interrupts

    PORTD |= (1<<1);
    delay_us(10); // Trigger the module to start
    PORTD &= ~(1<<1);
}

ISR(PCINT2_vect) {

    if (PIND & (1<<0)) { // Echo High; Start Measuring
        TCNT1 = 0;
    }
    else { // Echo Low; Measurement Complete

	uint16_t numuS = TCNT1;	 // Save Timer value
        hc04_echo_count = numuS;
        time_us = hc04_echo_count * 3.2; // (64/20e6 -> 3.2us per count)
        total_distance = time_us / 29.1; // (speed of sound is 29.1 us / cm)
        hc04_distance = total_distance / 2; // eliminate round trip; publish

	uint8_t oldSREG = SREG;

        // Retrigger to measure again

	PORTD |= (1<<1);
        delay_us(10); // Trigger the module for next time
	PORTD &= ~(1<<1);

	cli(); // Disable Global interrupts

	SREG = oldSREG;	// Enable interrupts
    }
}

